/**
 * Copyright 2013-2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @providesModule DOMPropertyOperations
 * @typechecks static-only
 */

"use strict";

var DOMProperty = require("./DOMProperty");
var quoteAttributeValueForBrowser = require("./quoteAttributeValueForBrowser");

function shouldIgnoreValue(name, value) {
  return value == null || DOMProperty.hasBooleanValue[name] && !value || DOMProperty.hasNumericValue[name] && isNaN(value) || DOMProperty.hasPositiveNumericValue[name] && value < 1 || DOMProperty.hasOverloadedBooleanValue[name] && value === false;
}

/**
 * Operations for dealing with DOM properties.
 */
var DOMPropertyOperations = {
  /**
   * Sets the value for a property on a node.
   *
   * @param {DOMElement} node
   * @param {string} name
   * @param {*} value
   */
  assignProperties: function assignProperties(props, name, value) {
    var hasChange;
    if (DOMProperty.isStandardName.hasOwnProperty(name) && DOMProperty.isStandardName[name]) {
      if (!shouldIgnoreValue(name, value) && DOMProperty.mustUseAttribute[name]) {
        // `setAttribute` with objects becomes only `[object]` in IE8/9,
        // ('' + value) makes it output the correct toString()-value.
        props.attributes[DOMProperty.getAttributeName[name]] = "" + value;
        hasChange = true;
      } else {
        var propName = DOMProperty.getPropertyName[name];
        // Must explicitly cast values for HAS_SIDE_EFFECTS-properties to the
        // property type before comparing; only `value` does and is string.
        if (!DOMProperty.hasSideEffects[name] || "" + props[propName] !== "" + value) {
          // Contrary to `setAttribute`, object properties are properly
          // `toString`ed by IE8/9.
          props[propName] = value;
          hasChange = true;
        }
      }
    } else if (DOMProperty.isCustomAttribute(name) && value != null) {
      props.attributes[name] = "" + value;
      hasChange = true;
    } else if ("production" !== process.env.NODE_ENV) {
      console.warn("unknow props: %s", name);
    }
    return hasChange;
  }
};

module.exports = DOMPropertyOperations;