'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = Component;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

var _util = require('./util');

var _ = _interopRequireWildcard(_util);

var _virtualDom = require('./virtual-dom');

var updateQueue = {
	updaters: [],
	isPending: false,
	add: function add(updater) {
		/*
   event bubbles from bottom-level to top-level
   reverse the updater order can merge some props and state and reduce the refresh times
   see Updater.update method below to know why
  */
		this.updaters.splice(0, 0, updater);
	},
	batchUpdate: function batchUpdate() {
		this.isPending = true;
		/*
    each updater.update may add new updater to updateQueue
    clear them with a loop
  */
		while (this.updaters.length) {
			var updaters = this.updaters;

			this.updaters = [];
			_.eachItem(updaters, triggerUpdate);
		}
		this.isPending = false;
	}
};
exports.updateQueue = updateQueue;
var triggerUpdate = function triggerUpdate(updater) {
	return updater.update();
};

function Updater(instance) {
	this.instance = instance;
	this.pendingStates = [];
	this.pendingCallbacks = [];
	this.isPending = false;
	this.nextProps = this.nextContext = null;
	this.clearCallbacks = this.clearCallbacks.bind(this);
}

Updater.prototype = {
	emitUpdate: function emitUpdate(nextProps, nextContext) {
		this.nextProps = nextProps;
		this.nextContext = nextContext;
		// receive nextProps!! should update immediately
		nextProps || !updateQueue.isPending ? this.update() : updateQueue.add(this);
	},
	update: function update() {
		var instance = this.instance;
		var pendingStates = this.pendingStates;
		var nextProps = this.nextProps;
		var nextContext = this.nextContext;

		if (nextProps || pendingStates.length > 0) {
			nextProps = nextProps || instance.props;
			nextContext = nextContext || instance.context;
			this.nextProps = this.nextContext = null;
			// merge the nextProps and nextState and update by one time
			shouldUpdate(instance, nextProps, this.getState(), nextContext, this.clearCallbacks);
		}
	},
	addState: function addState(nextState) {
		if (nextState) {
			this.pendingStates.push(nextState);
			if (!this.isPending) {
				this.emitUpdate();
			}
		}
	},
	replaceState: function replaceState(nextState) {
		var pendingStates = this.pendingStates;

		pendingStates.pop();
		// push special params to point out should replace state
		pendingStates.push([nextState]);
	},
	getState: function getState() {
		var instance = this.instance;
		var pendingStates = this.pendingStates;
		var state = instance.state;
		var props = instance.props;

		if (pendingStates.length) {
			state = _.extend({}, state);
			_.eachItem(pendingStates, function (nextState) {
				// replace state
				if (_.isArr(nextState)) {
					state = _.extend({}, nextState[0]);
					return;
				}
				if (_.isFn(nextState)) {
					nextState = nextState.call(instance, state, props);
				}
				_.extend(state, nextState);
			});
			pendingStates.length = 0;
		}
		return state;
	},
	clearCallbacks: function clearCallbacks() {
		var pendingCallbacks = this.pendingCallbacks;
		var instance = this.instance;

		if (pendingCallbacks.length > 0) {
			_.eachItem(pendingCallbacks, function (callback) {
				return callback.call(instance);
			});
			pendingCallbacks.length = 0;
		}
	},
	addCallback: function addCallback(callback) {
		if (_.isFn(callback)) {
			this.pendingCallbacks.push(callback);
		}
	}
};

function Component(props, context) {
	this.$updater = new Updater(this);
	this.$cache = { isMounted: false };
	this.props = props;
	this.state = {};
	this.refs = {};
	this.context = context || {};
}

var noop = _.noop;
Component.prototype = {
	constructor: Component,
	getChildContext: noop,
	componentWillUpdate: noop,
	componentDidUpdate: noop,
	componentWillReceiveProps: noop,
	componentWillMount: noop,
	componentDidMount: noop,
	componentWillUnmount: noop,
	shouldComponentUpdate: function shouldComponentUpdate(nextProps, nextState) {
		return true;
	},
	forceUpdate: function forceUpdate(callback) {
		var $updater = this.$updater;
		var $cache = this.$cache;
		var node = this.$node;
		var props = this.props;
		var state = this.state;
		var context = this.context;
		var vtree = this.vtree;

		if ($updater.isPending || !$cache.isMounted) {
			return;
		}
		var nextProps = $cache.props || props;
		var nextState = $cache.state || state;
		var nextContext = $cache.context || {};
		var parentContext = $cache.parentContext;
		var map = $cache.map;
		$cache.props = $cache.state = $cache.context = null;
		$updater.isPending = true;
		this.componentWillUpdate(nextProps, nextState, nextContext);
		this.state = nextState;
		this.props = nextProps;
		this.context = nextContext;
		var nextVtree = (0, _virtualDom.renderComponent)(this, parentContext);
		var newNode = vtree.updateTree(node, nextVtree, node.parentNode, nextVtree.context);
		(0, _virtualDom.clearDidMount)();
		$updater.isPending = false;
		if (newNode !== node) {
			map.remove(node);
			map.set(newNode, this);
		}
		this.vtree = nextVtree;
		this.$node = newNode;
		this.componentDidUpdate(props, state, context);
		if (_.isFn(callback)) {
			callback.call(this);
		}
		$updater.emitUpdate();
	},
	setState: function setState(nextState, callback) {
		var $updater = this.$updater;

		$updater.addCallback(callback);
		$updater.addState(nextState);
	},
	replaceState: function replaceState(nextState, callback) {
		var $updater = this.$updater;

		$updater.addCallback(callback);
		$updater.replaceState(nextState);
	},
	getDOMNode: function getDOMNode() {
		var node = this.$node;
		return node && node.tagName === 'NOSCRIPT' ? null : node;
	},
	isMounted: function isMounted() {
		return this.$cache.isMounted;
	}
};

var updatePropsAndState = function updatePropsAndState(component, props, state, context) {
	component.state = state;
	component.props = props;
	component.context = context || {};
};

exports.updatePropsAndState = updatePropsAndState;
var shouldUpdate = function shouldUpdate(component, nextProps, nextState, nextContext, callback) {
	var shouldComponentUpdate = component.shouldComponentUpdate(nextProps, nextState, nextContext);
	if (shouldComponentUpdate === false) {
		updatePropsAndState(component, nextProps, nextState, nextContext);
		return;
	}
	updatePropsAndState(component.$cache, nextProps, nextState, nextContext);
	component.forceUpdate(callback);
};
exports.shouldUpdate = shouldUpdate;