'use strict';

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _virtualDom = require('./virtual-dom');

var _component = require('./component');

var _refer = require('refer');

var _DOMPropertyOperations = require('./DOMPropertyOperations');

var _DOMProperty = require('./DOMProperty');

var _HTMLDOMPropertyConfig = require('./HTMLDOMPropertyConfig');

var _HTMLDOMPropertyConfig2 = _interopRequireDefault(_HTMLDOMPropertyConfig);

var _SVGDOMPropertyConfig = require('./SVGDOMPropertyConfig');

var _SVGDOMPropertyConfig2 = _interopRequireDefault(_SVGDOMPropertyConfig);

var _util = require('./util');

var isFn = _refer.types.isFn;
var isStr = _refer.types.isStr;
var isObj = _refer.types.isObj;
var isNum = _refer.types.isNum;

_DOMProperty.injection.injectDOMPropertyConfig(_HTMLDOMPropertyConfig2['default']);
_DOMProperty.injection.injectDOMPropertyConfig(_SVGDOMPropertyConfig2['default']);

var isUnitlessNumber = {
	animationIterationCount: true,
	boxFlex: true,
	boxFlexGroup: true,
	boxOrdinalGroup: true,
	columnCount: true,
	flex: true,
	flexGrow: true,
	flexPositive: true,
	flexShrink: true,
	flexNegative: true,
	flexOrder: true,
	fontWeight: true,
	lineClamp: true,
	lineHeight: true,
	opacity: true,
	order: true,
	orphans: true,
	tabSize: true,
	widows: true,
	zIndex: true,
	zoom: true,

	// SVG-related properties
	fillOpacity: true,
	stopOpacity: true,
	strokeDashoffset: true,
	strokeOpacity: true,
	strokeWidth: true
};

/**
 * @param {string} prefix vendor-specific prefix, eg: Webkit
 * @param {string} key style name, eg: transitionDuration
 * @return {string} style name prefixed with `prefix`, properly camelCased, eg:
 * WebkitTransitionDuration
 */
var prefixKey = function prefixKey(prefix, key) {
	return prefix + key.charAt(0).toUpperCase() + key.substring(1);
};

/**
 * Support style names that may come passed in prefixed by adding permutations
 * of vendor prefixes.
 */
var prefixes = ['Webkit', 'ms', 'Moz', 'O'];

// Using Object.keys here, or else the vanilla for-in loop makes IE8 go into an
// infinite loop, because it iterates over the newly added props too.
Object.keys(isUnitlessNumber).forEach(function (prop) {
	return prefixes.forEach(function (prefix) {
		return isUnitlessNumber[prefixKey(prefix, prop)] = isUnitlessNumber[prop];
	});
});

var RE_NUMBER = /^\d+(\.\d+)?$/;
var checkNum = function checkNum(obj) {
	return isNum(obj) || isStr(obj) && RE_NUMBER.test(obj);
};
var checkUnit = function checkUnit(style) {
	for (var key in style) {
		if (checkNum(style[key]) && !isUnitlessNumber[key]) {
			style[key] += 'px';
		}
	}
	return style;
};
var onchanging = null;
var checkEvent = function checkEvent(props) {
	var handle = props.onchange;
	if (isFn(handle)) {
		var onchange = function onchange(e) {
			onchanging = handle;
			handle.call(this, e);
			onchanging = null;
		};
		props.onchange = onchange;
		props.oninput = isFn(props.oninput) ? _util.pipe(props.oninput, onchange) : onchange;
		if (onchanging === handle && 'value' in props) {
			delete props.value;
		}
	}
	return props;
};
var isKey = function isKey(name) {
	return name === 'key';
};
var isEvent = function isEvent(name) {
	return /^on/.test(name);
};
var isStyle = function isStyle(name) {
	return name === 'style';
};
var isRef = function isRef(name) {
	return name === 'ref';
};
var assign = function assign(properties) {
	if (properties == null) {
		return properties;
	}
	var props = {
		attributes: {}
	};
	var hasChange = undefined;
	for (var _name in properties) {
		if (!properties.hasOwnProperty(_name)) {
			continue;
		}
		var value = properties[_name];
		if (isKey(_name)) {
			if (value != null) {
				props[_name] = value;
				hasChange = true;
			}
		} else if (isEvent(_name)) {
			if (isFn(value)) {
				props[_name.toLowerCase()] = value;
				hasChange = true;
			}
		} else if (isStyle(_name)) {
			if (isStr(value)) {
				props.attributes[_name] = value;
				hasChange = true;
			} else if (isObj(value)) {
				props[_name] = checkUnit(value);
				hasChange = true;
			}
		} else if (isRef(_name)) {
			if (isStr(value)) {
				var refKey = value;
				var refValue = value;
				props.attributes['data-refid'] = _component.collectRef(refKey, refValue);
				hasChange = true;
			}
		} else {
			hasChange = _DOMPropertyOperations.assignProperties(props, _name, value) || hasChange;
		}
	}
	return hasChange ? checkEvent(props) : null;
};

var getProps = function getProps(properties, children) {
	var length = children.length;

	properties = properties || {};
	if (length > 0) {
		properties.children = length === 1 ? children[0] : children;
	}
	return properties;
};

exports['default'] = function (tagName, properties) {
	for (var _len = arguments.length, children = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
		children[_key - 2] = arguments[_key];
	}

	var isComponent = isFn(tagName) && isFn(tagName.prototype.render);
	if (isComponent) {
		return new _component.Widget(tagName, getProps(properties, children));
	}
	if (isFn(tagName)) {
		tagName = tagName(getProps(properties, children));
	}
	var props = assign(properties);
	return _virtualDom.h.apply(undefined, [tagName, props].concat(children.filter(function (child) {
		return typeof child !== 'boolean';
	})));
};

module.exports = exports['default'];