// util
'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.Map = Map;

var _eventSystem = require('./event-system');

var isType = function isType(type) {
	return function (obj) {
		return obj != null && Object.prototype.toString.call(obj) === '[object ' + type + ']';
	};
};
exports.isType = isType;
var isObj = isType('Object');
exports.isObj = isObj;
var isStr = isType('String');
exports.isStr = isStr;
var isFn = isType('Function');
exports.isFn = isFn;
var isBln = isType('Boolean');
exports.isBln = isBln;
var isArr = Array.isArray || isType('Array');
exports.isArr = isArr;
var isUndefined = function isUndefined(obj) {
	return obj === undefined;
};
exports.isUndefined = isUndefined;
var isComponent = function isComponent(obj) {
	return obj && obj.prototype && 'forceUpdate' in obj.prototype;
};
exports.isComponent = isComponent;
var isStatelessComponent = function isStatelessComponent(obj) {
	return isFn(obj) && (!obj.prototype || !('forceUpdate' in obj.prototype));
};

exports.isStatelessComponent = isStatelessComponent;
var noop = function noop() {};
exports.noop = noop;
var identity = function identity(obj) {
	return obj;
};

exports.identity = identity;
var pipe = function pipe(fn1, fn2) {
	return function () {
		fn1.apply(this, arguments);
		return fn2.apply(this, arguments);
	};
};

exports.pipe = pipe;
var flattenChildren = function flattenChildren(list, iteratee, record) {
	record = record || { index: 0 };
	for (var i = 0, len = list.length; i < len; i++) {
		var item = list[i];
		if (isArr(item)) {
			flattenChildren(item, iteratee, record);
		} else if (!isUndefined(item) && !isBln(item)) {
			iteratee(item, record.index);
			record.index += 1;
		}
	}
};

exports.flattenChildren = flattenChildren;
var eachItem = function eachItem(list, iteratee) {
	for (var i = 0, len = list.length; i < len; i++) {
		iteratee(list[i], i);
	}
};

exports.eachItem = eachItem;
var findIndex = function findIndex(list, item, startIndex) {
	var i = startIndex > 0 ? startIndex : 0;
	for (var len = list.length; i < len; i++) {
		if (list[i] === item) {
			return i;
		}
	}
	return -1;
};

exports.findIndex = findIndex;
var mapValue = function mapValue(obj, iteratee) {
	for (var key in obj) {
		if (obj.hasOwnProperty(key)) {
			iteratee(obj[key], key);
		}
	}
};

exports.mapValue = mapValue;
var mapKey = function mapKey(oldObj, newObj, iteratee) {
	var keyMap = {};
	var key;
	for (key in oldObj) {
		if (oldObj.hasOwnProperty(key)) {
			keyMap[key] = true;
			iteratee(key);
		}
	}
	for (key in newObj) {
		if (newObj.hasOwnProperty(key) && keyMap[key] !== true) {
			iteratee(key);
		}
	}
};

exports.mapKey = mapKey;
var extend = function extend(target) {
	for (var i = 1, len = arguments.length; i < len; i++) {
		var source = arguments[i];
		if (source != null) {
			for (var key in source) {
				if (source.hasOwnProperty(key) && !isUndefined(source[key])) {
					target[key] = source[key];
				}
			}
		}
	}
	return target;
};

exports.extend = extend;

function Map() {
	this.store = [];
}

extend(Map.prototype, {
	get: function get(key) {
		var store = this.store;
		for (var i = 0, len = store.length; i < len; i++) {
			var item = store[i];
			if (item[0] === key) {
				return item[1];
			}
		}
	},
	set: function set(key, value) {
		var target = this.get(key);
		if (target) {
			target[1] = value;
		} else {
			this.store.push([key, value]);
		}
	},
	remove: function remove(key) {
		var store = this.store;
		for (var i = 0, len = store.length; i < len; i++) {
			var item = store[i];
			if (item[0] === key) {
				store.splice(i, 1);
				return item[1];
			}
		}
	}
});

var uid = 0;
var getUid = function getUid() {
	return ++uid;
};

exports.getUid = getUid;
var getChildren = function getChildren(children) {
	var childrenLen = children.length;
	if (childrenLen > 0) {
		if (childrenLen === 1) {
			children = children[0];
		}
		return children;
	}
};
var mergeProps = function mergeProps(props, children, defaultProps) {
	var result = extend({}, defaultProps, props);
	children = getChildren(children);
	if (!isUndefined(children)) {
		result.children = children;
	}
	return result;
};

exports.mergeProps = mergeProps;
var ignoreKeys = {
	key: true,
	ref: true,
	children: true
};
var EVENT_KEYS = /^on/i;
var isInnerHTMLKey = function isInnerHTMLKey(key) {
	return key === 'dangerouslySetInnerHTML';
};
var isStyleKey = function isStyleKey(key) {
	return key === 'style';
};
// Setting .type throws on non-<input> tags
var isTypeKey = function isTypeKey(key) {
	return key === 'type';
};

/*
  DOM Properties which are only getter
*/
var readOnlyProps = 'nodeName|nodeValue|nodeType|parentNode|childNodes|classList|firstChild|lastChild|previousSibling|previousElementSibling|nextSibling|nextElementSibling|attributes|ownerDocument|namespaceURI|localName|baseURI|prefix|length|specified|tagName|offsetTop|offsetLeft|offsetWidth|offsetHeight|offsetParent|scrollWidth|scrollHeight|clientTop|clientLeft|clientWidth|clientHeight|x|y';
var readOnlys = {};
eachItem(readOnlyProps.split('|'), function (key) {
	readOnlys[key] = true;
});
var setProp = function setProp(elem, key, value) {
	switch (true) {
		case ignoreKeys[key] === true:
			break;
		case EVENT_KEYS.test(key):
			(0, _eventSystem.addEvent)(elem, key, value);
			break;
		case isStyleKey(key):
			setStyle(elem, value);
			break;
		case isInnerHTMLKey(key):
			value && value.__html != null && (elem.innerHTML = value.__html);
			break;
		case key in elem && !isTypeKey(key):
			if (readOnlys[key] !== true && !(key === 'title' && value == null)) {
				elem[key] = value;
			}
			break;
		default:
			elem.setAttribute(key, '' + value);
	}
};
exports.setProp = setProp;
var setProps = function setProps(elem, props) {
	mapValue(props, function (value, key) {
		setProp(elem, key, value);
	});
};
exports.setProps = setProps;
var removeProps = function removeProps(elem, oldProps) {
	mapValue(oldProps, function (oldValue, key) {
		removeProp(elem, key, oldValue);
	});
};
exports.removeProps = removeProps;
var removeProp = function removeProp(elem, key, oldValue) {
	switch (true) {
		case ignoreKeys[key] === true:
			break;
		case EVENT_KEYS.test(key):
			(0, _eventSystem.removeEvent)(elem, key);
			break;
		case isStyleKey(key):
			removeStyle(elem, oldValue);
			break;
		case isInnerHTMLKey(key):
			elem.innerHTML = '';
			break;
		case !(key in elem) || isTypeKey(key):
			elem.removeAttribute(key);
			break;
		case isFn(oldValue):
			elem[key] = null;
			break;
		case isStr(oldValue):
			elem[key] = '';
			break;
		case isBln(oldValue):
			elem[key] = false;
			break;
		default:
			try {
				elem[key] = undefined;
				delete elem[key];
			} catch (e) {
				//pass
			}
	}
};

exports.removeProp = removeProp;
// use dom prop to compare new prop
var shouldUseDOMProp = {
	value: true,
	checked: true
};

var patchProps = function patchProps(elem, props, newProps) {
	if (props === newProps) {
		return;
	}
	if (!props && newProps) {
		setProps(elem, newProps);
		return;
	} else if (!newProps && props) {
		removeProps(elem, props);
		return;
	}

	mapKey(props, newProps, function (key) {
		if (ignoreKeys[key] === true) {
			return;
		}
		var value = newProps[key];
		var oldValue = shouldUseDOMProp[key] == true ? elem[key] : props[key];
		if (value === oldValue) {
			return;
		}
		if (isUndefined(value)) {
			removeProp(elem, key, oldValue);
			return;
		}
		if (isStyleKey(key)) {
			patchStyle(elem, oldValue, value);
		} else if (isInnerHTMLKey(key)) {
			var oldHtml = oldValue && oldValue.__html;
			var html = value && value.__html;
			if (html != null && html !== oldHtml) {
				elem.innerHTML = html;
			}
		} else {
			setProp(elem, key, value);
		}
	});
};

exports.patchProps = patchProps;
var removeStyle = function removeStyle(elem, style) {
	if (!isObj(style)) {
		return;
	}
	var elemStyle = elem.style;
	mapValue(style, function (_, key) {
		elemStyle[key] = '';
	});
};
exports.removeStyle = removeStyle;
var setStyle = function setStyle(elem, style) {
	if (!isObj(style)) {
		return;
	}
	var elemStyle = elem.style;
	mapValue(style, function (value, key) {
		setStyleValue(elemStyle, key, value);
	});
};
exports.setStyle = setStyle;
var patchStyle = function patchStyle(elem, style, newStyle) {
	if (style === newStyle) {
		return;
	}
	if (!newStyle && style) {
		removeStyle(elem, style);
	} else if (newStyle && !style) {
		setStyle(elem, newStyle);
	} else {
		var elemStyle = elem.style;
		mapKey(style, newStyle, function (key) {
			var value = newStyle[key];
			var oldValue = style[key];
			if (value !== oldValue) {
				setStyleValue(elemStyle, key, value);
			}
		});
	}
};

exports.patchStyle = patchStyle;
var isUnitlessNumber = {
	animationIterationCount: true,
	boxFlex: true,
	boxFlexGroup: true,
	boxOrdinalGroup: true,
	columnCount: true,
	flex: true,
	flexGrow: true,
	flexPositive: true,
	flexShrink: true,
	flexNegative: true,
	flexOrder: true,
	fontWeight: true,
	lineClamp: true,
	lineHeight: true,
	opacity: true,
	order: true,
	orphans: true,
	tabSize: true,
	widows: true,
	zIndex: true,
	zoom: true,

	// SVG-related properties
	fillOpacity: true,
	stopOpacity: true,
	strokeDashoffset: true,
	strokeOpacity: true,
	strokeWidth: true
};

var isUnitlessNumberWithPrefix = {};
var prefixes = ['Webkit', 'ms', 'Moz', 'O'];
var prefixKey = function prefixKey(prefix, key) {
	return prefix + key.charAt(0).toUpperCase() + key.substring(1);
};
mapValue(isUnitlessNumber, function (_, prop) {
	eachItem(prefixes, function (prefix) {
		return isUnitlessNumberWithPrefix[prefixKey(prefix, prop)] = true;
	});
});
mapValue(isUnitlessNumberWithPrefix, function (value, key) {
	isUnitlessNumber[key] = value;
});

var RE_NUMBER = /^-?\d+(\.\d+)?$/;
var setStyleValue = function setStyleValue(style, key, value) {
	if (value == null || isBln(value)) {
		value = '';
	}
	if (!isUnitlessNumber[key] && RE_NUMBER.test(value)) {
		style[key] = value + 'px';
	} else {
		style[key] = value;
	}
};

exports.setStyleValue = setStyleValue;
if (!Object.freeze) {
	Object.freeze = identity;
}