//types.js
'use strict';

exports.__esModule = true;
var isType = function isType(type) {
	return function (obj) {
		return obj != null && Object.prototype.toString.call(obj) === '[object ' + type + ']';
	};
};
exports.isType = isType;
var isObj = isType('Object');
exports.isObj = isObj;
var isStr = isType('String');
exports.isStr = isStr;
var isNum = isType('Number');
exports.isNum = isNum;
var isFn = isType('Function');
exports.isFn = isFn;
var isBln = isType('Boolean');
exports.isBln = isBln;
var isArr = Array.isArray || isType('Array');
exports.isArr = isArr;
var isThenable = function isThenable(obj) {
	return obj != null && isFn(obj.then);
};

exports.isThenable = isThenable;
var pipe = function pipe(fn1, fn2) {
	return function () {
		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		fn1.apply(this, args);
		return fn2.apply(this, args);
	};
};

exports.pipe = pipe;
var setProp = function setProp(elem, key, value) {
	switch (true) {
		case key === 'style':
			setStyle(elem, value);
			break;
		case /^on/.test(key):
			setEvent(elem, key.toLowerCase(), value);
			break;
		case key in elem:
			elem[key] = value;
			break;
		default:
			elem.setAttribute(key, value);
	}
};

exports.setProp = setProp;
var setProps = function setProps(elem, props) {
	Object.keys(props).forEach(function (key) {
		return setProp(elem, key, props[key]);
	});
};

exports.setProps = setProps;
var isEventKey = function isEventKey(key) {
	return /^on/.test(key);
};

exports.isEventKey = isEventKey;
var removeProp = function removeProp(elem, key) {
	var oldValue = elem[key];
	switch (true) {
		case isEventKey(key):
			removeEvent(elem, key);
			break;
		case isFn(oldValue):
			elem[key] = null;
			break;
		case isStr(oldValue):
			elem[key] = '';
			break;
		case isBln(oldValue):
			elem[key] = false;
			break;
		default:
			try {
				elem[key] = undefined;
				delete elem[key];
			} catch (e) {}
	}
};

exports.removeProp = removeProp;
var setEvent = function setEvent(elem, key, value) {
	key = key.toLowerCase();
	elem[key] = value;
	if (key === 'onchange' && !elem.oninput) {
		elem.oninput = value;
		value.oninput = true;
	}
};

exports.setEvent = setEvent;
var removeEvent = function removeEvent(elem, key) {
	key = key.toLowerCase();
	if (isFn(elem[key]) && elem[key].oninput) {
		elem.oninput = null;
	}
	elem[key] = null;
};

exports.removeEvent = removeEvent;
var removeStyle = function removeStyle(elem, style) {
	if (!isObj(style)) {
		return;
	}
	Object.keys(style).forEach(function (key) {
		return elem.style[key] = '';
	});
};

exports.removeStyle = removeStyle;
var setStyle = function setStyle(elem, style) {
	if (!isObj(style)) {
		return;
	}
	Object.keys(style).forEach(function (key) {
		setStyleValue(elem.style, key, style[key]);
	});
};

exports.setStyle = setStyle;
var isUnitlessNumber = {
	animationIterationCount: true,
	boxFlex: true,
	boxFlexGroup: true,
	boxOrdinalGroup: true,
	columnCount: true,
	flex: true,
	flexGrow: true,
	flexPositive: true,
	flexShrink: true,
	flexNegative: true,
	flexOrder: true,
	fontWeight: true,
	lineClamp: true,
	lineHeight: true,
	opacity: true,
	order: true,
	orphans: true,
	tabSize: true,
	widows: true,
	zIndex: true,
	zoom: true,

	// SVG-related properties
	fillOpacity: true,
	stopOpacity: true,
	strokeDashoffset: true,
	strokeOpacity: true,
	strokeWidth: true
};

/**
 * @param {string} prefix vendor-specific prefix, eg: Webkit
 * @param {string} key style name, eg: transitionDuration
 * @return {string} style name prefixed with `prefix`, properly camelCased, eg:
 * WebkitTransitionDuration
 */
var prefixKey = function prefixKey(prefix, key) {
	return prefix + key.charAt(0).toUpperCase() + key.substring(1);
};

/**
 * Support style names that may come passed in prefixed by adding permutations
 * of vendor prefixes.
 */
var prefixes = ['Webkit', 'ms', 'Moz', 'O'];

// Using Object.keys here, or else the vanilla for-in loop makes IE8 go into an
// infinite loop, because it iterates over the newly added props too.
Object.keys(isUnitlessNumber).forEach(function (prop) {
	return prefixes.forEach(function (prefix) {
		return isUnitlessNumber[prefixKey(prefix, prop)] = isUnitlessNumber[prop];
	});
});

var RE_NUMBER = /^\d+(\.\d+)?$/;
var setStyleValue = function setStyleValue(style, key, value) {
	if (RE_NUMBER.test(value) && !isUnitlessNumber[key]) {
		style[key] = value + 'px';
	} else {
		style[key] = value;
	}
};
exports.setStyleValue = setStyleValue;

//pass